from django.db import models
from django.core.validators import RegexValidator, MaxValueValidator


class IPAddresses(models.Model):
    """Модель данных для хранения IP адрессов (нужно для подсчета количества просмотров)"""
    ipAddressRegex = RegexValidator(regex = r"^((25[0-5]|(2[0-4]|1\d|[1-9]|)\d)\.?\b){4}$")
    ipAddress = models.CharField(validators=[ipAddressRegex], max_length=15, verbose_name="IP адрес")

    def __str__(self):
        return self.ipAddress

    class Meta:
        verbose_name = "IP адрес"
        verbose_name_plural = "IP адреса"


class Apartament(models.Model):
    """Модель данных квартиры"""
    price = models.IntegerField(validators=[MaxValueValidator(450000)], verbose_name="Стоимость аренды квартиры за месяц")
    lastPrice = models.IntegerField(validators=[MaxValueValidator(450000)], default = 0, verbose_name="Прошлая стоимость аренды квартиры за месяц")
    bail = models.IntegerField(validators=[MaxValueValidator(450000)], verbose_name="Стоимость залога квартиры")
    agencyCommission = models.IntegerField(validators=[MaxValueValidator(450000)], verbose_name="Коммисия агенству")
    utilitiesPrice = models.CharField(max_length=100, verbose_name="Стоимость коммунальных услуг")
    minimumLeasePeriod = models.IntegerField(validators=[MaxValueValidator(60)], verbose_name="Минимальный срок аренды в месяцах")

    address = models.CharField(max_length=100, verbose_name="Адрес квартиры")
    description = models.TextField(max_length=300, verbose_name="Описание квартиры")

    perimetrs = models.FloatField(validators=[MaxValueValidator(250)], verbose_name="Количество квадратных метров")
    rooms = models.IntegerField(validators=[MaxValueValidator(10)], verbose_name="Количество комнат")
    ceilingHeight = models.FloatField(validators=[MaxValueValidator(5)], verbose_name="Высота потолков в метрах")
    floor = models.IntegerField(validators=[MaxValueValidator(50)], verbose_name="Этаж квартиры")
    floorHouse = models.IntegerField(validators=[MaxValueValidator(50)], verbose_name="Количество этажей в доме")

    phoneNumberRegex = RegexValidator(regex = r"^\+?1?\d{8,15}$")
    phoneNumber = models.CharField(validators=[phoneNumberRegex], max_length=16, unique=True, verbose_name="Номер телефона для связи")

    timeToBus = models.IntegerField(validators=[MaxValueValidator(60)], verbose_name="Время до ближайшей автобусной остановки в минутах")
    timeToTrain = models.IntegerField(validators=[MaxValueValidator(60)], verbose_name="Время до ближайшей Ж/Д станции в минутах")

    """Параметры квартиры"""
    isFurniture = models.BooleanField(verbose_name="Имеется мебель в квартире")
    isAnimal = models.BooleanField(verbose_name="Можно с питомцем")
    isTelevision = models.BooleanField(verbose_name="Имеется телевизор в квартире")
    isChild = models.BooleanField(verbose_name="Можно с детьми")
    isInternet = models.BooleanField(verbose_name="Проведен интернет в квартире")
    isBathroom = models.BooleanField(verbose_name="Имеется санузел в квартире")
    isRefrigerator = models.BooleanField(verbose_name="Имеется холодильник в квартире")
    isWasher = models.BooleanField(verbose_name="Имеется стиральная машинка в квартире")
    isAirConditioning = models.BooleanField(verbose_name="Имеется кондиционер в квартире")
    isFreshRepair = models.BooleanField(verbose_name="Недавно был ремонт в квартире")

    """Параметры дома"""
    isElevator = models.BooleanField(verbose_name="Имеется лифт в доме")
    isParking = models.BooleanField(verbose_name="Имеется закрытая парковка у дома")
    isGarbageChute = models.BooleanField(verbose_name="Имеется мусоропровод в доме")
    isConcierge = models.BooleanField(verbose_name="Имеется консьерж в доме")

    views = models.ManyToManyField(IPAddresses, related_name="post_views", blank=True)
    
    dateCreate = models.DateField(auto_now_add=True, editable=False, verbose_name="Дата добавления квартиры")

    def total_views(self):
        return self.views.count()

    class Meta:
        verbose_name = "Квартира"
        verbose_name_plural = "Квартиры"
        ordering = ['-dateCreate']


class Photo(models.Model):
    """Модель данных фотографий квартир"""
    apartament = models.ForeignKey(Apartament, on_delete = models.CASCADE, verbose_name="Квартира")
    image = models.ImageField(upload_to='photos/%Y/%m/%d', verbose_name="Фотография")

    class Meta:
        verbose_name = "Фотография"
        verbose_name_plural = "Фотографии"

class User(models.Model):
    """Тестовая модель данных для избранных и сравнения"""
    favorites_apartaments = models.CharField(max_length=100, verbose_name="Избранные квартиры")
    apartaments_for_comparison = models.CharField(max_length=100, verbose_name="Квартиры для сравнения")

    class Meta:
        verbose_name = "Пользователь"
        verbose_name_plural = "Пользователи"